@@ -15,6 +15,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/jpillora/backoff"
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common/buildlogger"
@@ -310,7 +311,8 @@ func (b *Build) getCustomBuildDir(rootDir, dir string, customBuildDirEnabled, sh
 	}
 
 	if !customBuildDirEnabled {
-		return "", MakeBuildError("setting GIT_CLONE_PATH is not allowed, enable `custom_build_dir` feature")
+		return "", MakeBuildError("setting GIT_CLONE_PATH is not allowed, enable `custom_build_dir` feature").
+			WithFailureReason(ConfigurationError)
 	}
 
 	// See: https://gitlab.com/gitlab-org/gitlab-runner/-/issues/25913
@@ -319,7 +321,8 @@ func (b *Build) getCustomBuildDir(rootDir, dir string, customBuildDirEnabled, sh
 		return "", &BuildError{Inner: err}
 	}
 	if strings.HasPrefix(relDir, "..") {
-		return "", MakeBuildError("the GIT_CLONE_PATH=%q has to be within %q", dir, rootDir)
+		return "", MakeBuildError("the GIT_CLONE_PATH=%q has to be within %q", dir, rootDir).
+			WithFailureReason(ConfigurationError)
 	}
 
 	return path.Clean(dir), nil
@@ -330,11 +333,11 @@ func (b *Build) StartBuild(
 	customBuildDirEnabled, sharedDir, safeDirectoryCheckout bool,
 ) error {
 	if rootDir == "" {
-		return MakeBuildError("the builds_dir is not configured")
+		return MakeBuildError("the builds_dir is not configured").WithFailureReason(ConfigurationError)
 	}
 
 	if cacheDir == "" {
-		return MakeBuildError("the cache_dir is not configured")
+		return MakeBuildError("the cache_dir is not configured").WithFailureReason(ConfigurationError)
 	}
 
 	b.SafeDirectoryCheckout = safeDirectoryCheckout
@@ -491,6 +494,26 @@ func (b *Build) executeArchiveCache(ctx context.Context, state error, executor E
 	return b.executeStage(ctx, BuildStageArchiveOnFailureCache, executor)
 }
 
+func asRunnerSystemFailure(inner error) error { return asBuildError(inner, RunnerSystemFailure) }
+
+func asBuildError(inner error, reason JobFailureReason) error {
+	if inner == nil {
+		return nil
+	}
+
+	// If there's already a BuildError in the chain, leave it as it is...
+	var be *BuildError
+	if errors.As(inner, &be) {
+		return inner
+	}
+
+	return &BuildError{
+		Inner:         inner,
+		FailureReason: reason,
+		ExitCode:      1,
+	}
+}
+
 //nolint:gocognit
 func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Executor) error {
 	// track job start and create referees
@@ -500,20 +523,27 @@ func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Exec
 	// Prepare stage
 	err := b.executeStage(ctx, BuildStagePrepare, executor)
 	if err != nil {
-		return fmt.Errorf(
+		return asRunnerSystemFailure(fmt.Errorf(
 			"prepare environment: %w. "+
-				"Check https://docs.gitlab.com/runner/shells/#shell-profile-loading for more information",
-			err,
-		)
+				"Check https://docs.gitlab.com/runner/shells/#shell-profile-loading for more information", err))
 	}
 
-	err = b.attemptGetSourcesStage(ctx, executor, b.GetGetSourcesAttempts())
+	err = asRunnerSystemFailure(b.attemptExecuteStage(ctx, BuildStageGetSources, executor, b.GetGetSourcesAttempts(), func(attempt int) error {
+		if attempt == 1 {
+			// If GetSources fails we delete all tracked and untracked files. This is
+			// because Git's submodule support has various bugs that cause fetches to
+			// fail if submodules have changed.
+			return b.executeStage(ctx, BuildStageClearWorktree, executor)
+		}
+
+		return nil
+	}))
 
 	if err == nil {
-		err = b.attemptExecuteStage(ctx, BuildStageRestoreCache, executor, b.GetRestoreCacheAttempts())
+		err = asRunnerSystemFailure(b.attemptExecuteStage(ctx, BuildStageRestoreCache, executor, b.GetRestoreCacheAttempts(), nil))
 	}
 	if err == nil {
-		err = b.attemptExecuteStage(ctx, BuildStageDownloadArtifacts, executor, b.GetDownloadArtifactsAttempts())
+		err = asRunnerSystemFailure(b.attemptExecuteStage(ctx, BuildStageDownloadArtifacts, executor, b.GetDownloadArtifactsAttempts(), nil))
 	}
 
 	//nolint:nestif
@@ -581,9 +611,9 @@ func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Exec
 		}
 	}
 
-	archiveCacheErr := b.executeArchiveCache(ctx, err, executor)
+	archiveCacheErr := asRunnerSystemFailure(b.executeArchiveCache(ctx, err, executor))
 
-	artifactUploadErr := b.executeUploadArtifacts(ctx, err, executor)
+	artifactUploadErr := asRunnerSystemFailure(b.executeUploadArtifacts(ctx, err, executor))
 
 	// track job end and execute referees
 	endTime := time.Now()
@@ -673,45 +703,44 @@ func (b *Build) executeUploadReferees(ctx context.Context, startTime, endTime ti
 	}
 }
 
-func (b *Build) attemptGetSourcesStage(
-	ctx context.Context,
-	executor Executor,
-	attempts int,
-) (err error) {
-	if attempts < 1 || attempts > 10 {
-		return fmt.Errorf("number of attempts out of the range [1, 10] for stage: %s", BuildStageGetSources)
-	}
-	for attempt := 0; attempt < attempts; attempt++ {
-		if attempt == 1 {
-			// If GetSources fails we delete all tracked and untracked files. This is
-			// because Git's submodule support has various bugs that cause fetches to
-			// fail if submodules have changed.
-			if err = b.executeStage(ctx, BuildStageClearWorktree, executor); err != nil {
-				continue
-			}
-		}
-		if err = b.executeStage(ctx, BuildStageGetSources, executor); err == nil {
-			return
-		}
-	}
-	return
-}
-
 func (b *Build) attemptExecuteStage(
 	ctx context.Context,
 	buildStage BuildStage,
 	executor Executor,
 	attempts int,
-) (err error) {
+	retryCallback func(attempt int) error,
+) error {
 	if attempts < 1 || attempts > 10 {
 		return fmt.Errorf("number of attempts out of the range [1, 10] for stage: %s", buildStage)
 	}
+
+	retry := backoff.Backoff{
+		Min:    5 * time.Second,
+		Max:    5 * time.Minute,
+		Jitter: true,
+		Factor: 1.5,
+	}
+
+	var err error
 	for attempt := 0; attempt < attempts; attempt++ {
+		if retryCallback != nil {
+			if err = retryCallback(attempt); err != nil {
+				continue
+			}
+		}
+
 		if err = b.executeStage(ctx, buildStage, executor); err == nil {
-			return
+			return nil
+		}
+
+		if b.IsFeatureFlagOn(featureflags.UseExponentialBackoffStageRetry) {
+			duration := retry.Duration()
+			b.logger.Infoln(fmt.Sprintf("Retrying in %v", duration))
+			time.Sleep(duration)
 		}
 	}
-	return
+
+	return err
 }
 
 func (b *Build) GetBuildTimeout() time.Duration {
@@ -782,8 +811,6 @@ func (b *Build) run(ctx context.Context, trace JobTrace, executor Executor) (err
 		buildFinish <- b.executeScript(runContext, trace, executor)
 	}()
 
-	defer b.ensureFinishedAt()
-
 	// Wait for signals: cancel, timeout, abort or finish
 	b.Log().Debugln("Waiting for signals...")
 	select {
@@ -826,6 +853,8 @@ func (b *Build) run(ctx context.Context, trace JobTrace, executor Executor) (err
 	runCancel()
 	b.waitForBuildFinish(buildFinish, WaitForBuildFinishTimeout)
 
+	b.ensureFinishedAt()
+
 	return err
 }
 
