@@ -13,7 +13,7 @@ import (
 	"os"
 	"path/filepath"
 	"strconv"
-	"syscall"
+	"strings"
 	"testing"
 	"time"
 
@@ -397,7 +397,7 @@ func TestJobFailure(t *testing.T) {
 	executor.On("Cleanup").Once()
 
 	// Set up a failing a build script
-	thrownErr := &BuildError{Inner: errors.New("test error"), ExitCode: 1}
+	thrownErr := errors.Join(&BuildError{Inner: errors.New("test error"), ExitCode: 1})
 	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", mock.Anything).Return(thrownErr).Times(3)
@@ -526,7 +526,11 @@ func TestGetSourcesRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
-	executor.On("Finish", errors.New("build fail")).Once()
+	executor.On("Finish",
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "build fail")
+		})).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
 	build.Variables = append(build.Variables, JobVariable{Key: "GET_SOURCES_ATTEMPTS", Value: "3"})
@@ -551,7 +555,11 @@ func TestArtifactDownloadRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
-	executor.On("Finish", errors.New("build fail")).Once()
+	executor.On("Finish",
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "build fail")
+		})).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
 	build.Variables = append(build.Variables, JobVariable{Key: "ARTIFACT_DOWNLOAD_ATTEMPTS", Value: "3"})
@@ -578,7 +586,11 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageArchiveOnSuccessCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(errors.New("upload fail")).Once()
 	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
-	executor.On("Finish", errors.New("upload fail")).Once()
+	executor.On("Finish",
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "upload fail")
+		})).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
 	successfulBuild := build.JobResponse
@@ -638,7 +650,11 @@ func TestUploadArtifactsOnArchiveCacheFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageArchiveOnSuccessCache)).Return(errors.New("cache failure")).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
-	executor.On("Finish", errors.New("cache failure")).Once()
+	executor.On("Finish",
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "cache failure")
+		})).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
 	err := build.Run(&Config{}, &Trace{Writer: os.Stdout})
@@ -661,7 +677,11 @@ func TestRestoreCacheRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
-	executor.On("Finish", errors.New("build fail")).Once()
+	executor.On("Finish",
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "build fail")
+		})).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
 	build.Variables = append(build.Variables, JobVariable{Key: "RESTORE_CACHE_ATTEMPTS", Value: "3"})
@@ -685,7 +705,10 @@ func TestRunWrongAttempts(t *testing.T) {
 		Return(errors.New("number of attempts out of the range [1, 10] for stage: get_sources"))
 	executor.On(
 		"Finish",
-		errors.New("number of attempts out of the range [1, 10] for stage: get_sources"),
+		mock.MatchedBy(func(err error) bool {
+			return errors.Is(err, &BuildError{FailureReason: RunnerSystemFailure}) &&
+				strings.Contains(err.Error(), "number of attempts out of the range [1, 10] for stage: get_sources")
+		}),
 	)
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -3183,88 +3206,3 @@ func TestBuildDurationsAndBoundaryTimes(t *testing.T) {
 	assert.Equal(t, finishedAt1, finishedAt2, "FinishedAt() should not change")
 	assert.Equal(t, startedAt1, startedAt2, "StartedAt() should not change")
 }
-
-func TestBuild_runCallsEnsureFinishedAt(t *testing.T) {
-	tests := map[string]struct {
-		mockRun     func(t *testing.T, cancelFn func(error), interrupt chan os.Signal, executor *MockExecutor)
-		assertError func(t *testing.T, err error)
-	}{
-		"succeeded job": {
-			mockRun: func(t *testing.T, _ func(error), _ chan os.Signal, executor *MockExecutor) {
-				executor.EXPECT().Run(mock.Anything).Return(nil)
-			},
-		},
-		"failed job": {
-			mockRun: func(t *testing.T, _ func(error), _ chan os.Signal, executor *MockExecutor) {
-				executor.EXPECT().Run(mock.Anything).Return(assert.AnError)
-			},
-			assertError: func(t *testing.T, err error) {
-				assert.ErrorIs(t, err, assert.AnError)
-			},
-		},
-		"canceled job": {
-			mockRun: func(t *testing.T, cancelFn func(error), _ chan os.Signal, executor *MockExecutor) {
-				executor.EXPECT().Run(mock.Anything).RunAndReturn(func(command ExecutorCommand) error {
-					cancelFn(assert.AnError)
-					time.Sleep(10 * time.Millisecond)
-					return nil
-				}).Once()
-				executor.EXPECT().Run(mock.Anything).Return(nil)
-			},
-			assertError: func(t *testing.T, err error) {
-				assert.ErrorIs(t, err, assert.AnError)
-			},
-		},
-		"interrupted runner process": {
-			mockRun: func(t *testing.T, _ func(error), interrupt chan os.Signal, executor *MockExecutor) {
-				executor.EXPECT().Run(matchBuildStage(BuildStagePrepare)).RunAndReturn(func(command ExecutorCommand) error {
-					interrupt <- syscall.SIGHUP
-					time.Sleep(10 * time.Millisecond)
-					return nil
-				}).Once()
-				executor.EXPECT().Run(mock.Anything).Return(nil)
-			},
-			assertError: func(t *testing.T, err error) {
-				var eerr *BuildError
-				if assert.ErrorAs(t, err, &eerr) {
-					assert.Equal(t, RunnerSystemFailure, eerr.FailureReason)
-					assert.Contains(t, eerr.Inner.Error(), "aborted: hangup")
-				}
-			},
-		},
-	}
-
-	for tn, tt := range tests {
-		t.Run(tn, func(t *testing.T) {
-			rc := new(RunnerConfig)
-			rc.RunnerSettings.Executor = t.Name()
-
-			interrupt := make(chan os.Signal, 1)
-
-			build, err := NewBuild(JobResponse{}, rc, interrupt, nil)
-			require.NoError(t, err)
-
-			ctx, cancelFn := context.WithCancelCause(context.Background())
-			defer cancelFn(nil)
-
-			require.Zero(t, build.finishedAt)
-
-			executor := NewMockExecutor(t)
-			executor.EXPECT().Shell().Return(&ShellScriptInfo{Shell: "script-shell"})
-			require.NotNil(t, tt.mockRun)
-			tt.mockRun(t, cancelFn, interrupt, executor)
-
-			trace := NewMockJobTrace(t)
-			trace.EXPECT().SetCancelFunc(mock.AnythingOfType("context.CancelFunc")).Maybe()
-
-			err = build.run(ctx, trace, executor)
-			if tt.assertError != nil {
-				tt.assertError(t, err)
-			} else {
-				assert.NoError(t, err)
-			}
-
-			assert.NotZero(t, build.finishedAt)
-		})
-	}
-}
