@@ -301,7 +301,7 @@ func (e *executor) createService(
 	linkNames []string,
 ) (*types.Container, error) {
 	if service == "" {
-		return nil, common.MakeBuildError("invalid service image name: %s", definition.Name)
+		return nil, common.MakeBuildError("invalid service image name: %s", definition.Name).WithFailureReason(common.ConfigurationError)
 	}
 
 	if e.volumesManager == nil {
@@ -855,7 +855,7 @@ func (e *executor) startAndWatchContainer(ctx context.Context, id string, input
 	if id == e.buildContainerID && e.Build.UseNativeSteps() {
 		request, err := steps.NewRequest(e.Build)
 		if err != nil {
-			return common.MakeBuildError("creating steps request: %w", err)
+			return common.MakeBuildError("creating steps request: %w", err).WithFailureReason(common.ConfigurationError)
 		}
 		dockerExec = exec.NewStepsDocker(e.Context, e.client, e.waiter, e.Build.Log(), request)
 	}
@@ -917,7 +917,7 @@ func (e *executor) removeContainer(ctx context.Context, id string) error {
 func (e *executor) disconnectNetwork(ctx context.Context, id string) {
 	e.BuildLogger.Debugln("Disconnecting container", id, "from networks")
 
-	netList, err := e.client.NetworkList(ctx, types.NetworkListOptions{})
+	netList, err := e.client.NetworkList(ctx, network.ListOptions{})
 	if err != nil {
 		e.BuildLogger.Debugln("Can't get network list. ListNetworks exited with", err)
 		return
@@ -1335,7 +1335,7 @@ func (e *executor) prepareHelperImage() (helperimage.Info, error) {
 
 func (e *executor) prepareBuildsDir(options common.ExecutorPrepareOptions) error {
 	if e.volumeParser == nil {
-		return common.MakeBuildError("missing volume parser")
+		return common.MakeBuildError("missing volume parser").WithFailureReason(common.RunnerSystemFailure)
 	}
 
 	isHostMounted, err := volumes.IsHostMountedVolume(e.volumeParser, e.RootDir(), options.Config.Docker.Volumes...)
@@ -1447,7 +1447,7 @@ func shouldIgnoreDockerError(err error, isFuncs ...func(error) bool) bool {
 
 func (e *executor) execScriptOnContainer(ctx context.Context, containerID string, script ...string) (err error) {
 	action := ""
-	execConfig := types.ExecConfig{
+	execConfig := container.ExecOptions{
 		Tty:          false,
 		AttachStderr: true,
 		AttachStdout: true,
@@ -1466,7 +1466,7 @@ func (e *executor) execScriptOnContainer(ctx context.Context, containerID string
 		return err
 	}
 
-	resp, err := e.client.ContainerExecAttach(ctx, exec.ID, types.ExecStartCheck{})
+	resp, err := e.client.ContainerExecAttach(ctx, exec.ID, container.ExecStartOptions{})
 	if err != nil {
 		action = "Failed to exec attach to container:"
 		return err
