@@ -17,6 +17,7 @@ import (
 
 	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
+	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/system"
 	"github.com/docker/docker/api/types/volume"
@@ -870,7 +871,7 @@ func prepareTestDockerConfiguration(
 	c.On("ImagePullBlocking", mock.Anything, expectedPullImage, mock.Anything).
 		Return(nil).Once()
 	c.On("NetworkList", mock.Anything, mock.Anything).
-		Return([]types.NetworkResource{}, nil).Once()
+		Return([]network.Summary{}, nil).Once()
 	c.On("ContainerRemove", mock.Anything, mock.Anything, mock.Anything).
 		Return(nil).Once()
 
@@ -1197,6 +1198,7 @@ func TestDockerContainerLabelsSetting(t *testing.T) {
 			"com.gitlab.gitlab-runner.job.ref":         "",
 			"com.gitlab.gitlab-runner.job.sha":         "",
 			"com.gitlab.gitlab-runner.job.url":         "/-/jobs/0",
+			"com.gitlab.gitlab-runner.job.timeout":     "2h0m0s",
 			"com.gitlab.gitlab-runner.managed":         "true",
 			"com.gitlab.gitlab-runner.pipeline.id":     "",
 			"com.gitlab.gitlab-runner.project.id":      "0",
@@ -1739,7 +1741,7 @@ func TestDockerCreateNetwork(t *testing.T) {
 					Return(container.NetworkMode("test"), nil).
 					Once()
 				nm.On("Inspect", mock.Anything).
-					Return(types.NetworkResource{}, nil).
+					Return(network.Inspect{}, nil).
 					Once()
 				nm.On("Cleanup", mock.Anything).
 					Return(nil).
@@ -1754,7 +1756,7 @@ func TestDockerCreateNetwork(t *testing.T) {
 					Return(container.NetworkMode("test"), nil).
 					Once()
 				nm.On("Inspect", mock.Anything).
-					Return(types.NetworkResource{}, nil).
+					Return(network.Inspect{}, nil).
 					Once()
 				nm.On("Cleanup", mock.Anything).
 					Return(nil).
@@ -1779,7 +1781,7 @@ func TestDockerCreateNetwork(t *testing.T) {
 					Return(container.NetworkMode("test"), nil).
 					Once()
 				nm.On("Inspect", mock.Anything).
-					Return(types.NetworkResource{}, testErr).
+					Return(network.Inspect{}, testErr).
 					Once()
 			},
 			expectedCleanError: nil,
@@ -1789,7 +1791,7 @@ func TestDockerCreateNetwork(t *testing.T) {
 			networkPerBuild:      "true",
 			clientAssertions: func(c *docker.MockClient) {
 				c.On("NetworkList", mock.Anything, mock.Anything).
-					Return([]types.NetworkResource{}, nil).
+					Return([]network.Summary{}, nil).
 					Once()
 				c.On("ContainerRemove", mock.Anything, mock.Anything, mock.Anything).
 					Return(testErr).
@@ -1801,8 +1803,8 @@ func TestDockerCreateNetwork(t *testing.T) {
 					Once()
 				nm.On("Inspect", mock.Anything).
 					Return(
-						types.NetworkResource{
-							Containers: map[string]types.EndpointResource{
+						network.Inspect{
+							Containers: map[string]network.EndpointResource{
 								"abc": {},
 							},
 						},
@@ -1823,7 +1825,7 @@ func TestDockerCreateNetwork(t *testing.T) {
 					Return(container.NetworkMode("test"), nil).
 					Once()
 				nm.On("Inspect", mock.Anything).
-					Return(types.NetworkResource{}, nil).
+					Return(network.Inspect{}, nil).
 					Once()
 				nm.On("Cleanup", mock.Anything).
 					Return(testErr).
@@ -1963,7 +1965,7 @@ func TestCheckOSType(t *testing.T) {
 
 func TestHelperImageRegistry(t *testing.T) {
 	tests := map[string]struct {
-		build *common.Build
+		config *common.DockerConfig
 		// We only validate the name because we only care if the right image is
 		// used. We don't want to end up having this test as a "spellcheck" to
 		// make sure tags and commands are generated correctly since that is
@@ -1972,34 +1974,12 @@ func TestHelperImageRegistry(t *testing.T) {
 		expectedHelperImageName string
 	}{
 		"Default helper image": {
-			build: &common.Build{
-				JobResponse: common.JobResponse{
-					Image: common.Image{
-						Name: "test",
-					},
-				},
-				Runner: &common.RunnerConfig{
-					RunnerSettings: common.RunnerSettings{
-						Docker: &common.DockerConfig{},
-					},
-				},
-			},
+			config:                  &common.DockerConfig{},
 			expectedHelperImageName: helperimage.GitLabRegistryName,
 		},
 		"helper image overridden still use default helper image in prepare": {
-			build: &common.Build{
-				JobResponse: common.JobResponse{
-					Image: common.Image{
-						Name: "test",
-					},
-				},
-				Runner: &common.RunnerConfig{
-					RunnerSettings: common.RunnerSettings{
-						Docker: &common.DockerConfig{
-							HelperImage: "private.registry.com/helper",
-						},
-					},
-				},
+			config: &common.DockerConfig{
+				HelperImage: "private.registry.com/helper",
 			},
 			// We expect the default image to still be chosen since the check of
 			// the override happens at a later stage.
@@ -2018,20 +1998,16 @@ func TestHelperImageRegistry(t *testing.T) {
 				},
 			}
 
-			prepareOptions := common.ExecutorPrepareOptions{
-				Config: &common.RunnerConfig{
-					RunnerSettings: common.RunnerSettings{
-						Docker: tt.build.Runner.Docker,
-					},
-				},
-				Build:   tt.build,
-				Context: context.Background(),
+			e.Build = &common.Build{}
+			e.info = system.Info{
+				OSType: "linux",
 			}
+			e.Config.Docker = tt.config
 
-			err := e.Prepare(prepareOptions)
+			helperImageInfo, err := e.prepareHelperImage()
 			require.NoError(t, err)
 
-			assert.Equal(t, tt.expectedHelperImageName, e.helperImageInfo.Name)
+			assert.Equal(t, tt.expectedHelperImageName, helperImageInfo.Name)
 		})
 	}
 }
@@ -2060,7 +2036,7 @@ func TestLocalHelperImage(t *testing.T) {
 					mock.Anything,
 					mock.Anything,
 					helperimage.GitLabRegistryName,
-					types.ImageImportOptions{
+					image.ImportOptions{
 						Tag: "x86_64-latest",
 						Changes: []string{
 							`ENTRYPOINT ["/usr/bin/dumb-init", "/entrypoint"]`,
@@ -2134,7 +2110,7 @@ func TestLocalHelperImage(t *testing.T) {
 				c.On(
 					"ImageImportBlocking",
 					mock.Anything,
-					mock.MatchedBy(func(source types.ImageImportSource) bool {
+					mock.MatchedBy(func(source image.ImportSource) bool {
 						return assert.IsType(t, new(os.File), source.Source) &&
 							assert.Equal(
 								t,
@@ -2175,7 +2151,7 @@ func TestLocalHelperImage(t *testing.T) {
 				c.On(
 					"ImageImportBlocking",
 					mock.Anything,
-					mock.MatchedBy(func(source types.ImageImportSource) bool {
+					mock.MatchedBy(func(source image.ImportSource) bool {
 						return assert.IsType(t, new(os.File), source.Source) &&
 							assert.Equal(
 								t,
@@ -2217,7 +2193,7 @@ func TestLocalHelperImage(t *testing.T) {
 					mock.Anything,
 					mock.Anything,
 					true,
-				).Return(types.ImageLoadResponse{JSON: true, Body: io.NopCloser(strings.NewReader(`{"stream": "Loaded image ID: 1234"}`))}, nil)
+				).Return(image.LoadResponse{JSON: true, Body: io.NopCloser(strings.NewReader(`{"stream": "Loaded image ID: 1234"}`))}, nil)
 
 				c.On(
 					"ImageTag",
@@ -2429,7 +2405,7 @@ func TestExpandingDockerImageWithImagePullPolicyNever(t *testing.T) {
 	assert.Contains(
 		t,
 		err.Error(),
-		fmt.Sprintf(common.IncompatiblePullPolicy, "[never]", "GitLab pipeline config", "[always]"),
+		fmt.Sprintf("pull_policy (%v) defined in %s is not one of the allowed_pull_policies (%v)", "[never]", "GitLab pipeline config", "[always]"),
 	)
 }
 
